/* Shortcut Launcher v1.0
 *
 * 
 * Copyright (C) 2006 BoxiT Software.
 *
 * ShortcutManagement.js
 *
 * Requires Yahoo! Widget Engine 3 or later
 */
  
function populateShortcuts()
{
  // Check that the xml file exists
  if (filesystem.itemExists(xmlPath))
  {
    // Xml file exists so we can load the data
    var data = filesystem.readFile(xmlPath);
    var xml = XMLDOM.parse(data);
    
    // First we query the xml to get the shortcut items
    var results = xml.evaluate("Shortcuts/Shortcut");
  
    if (results.length == 0)
    {
      return;
    }
    
    for (i = 0; i < results.length; i++)
    {
      var node = results.item(i);
      shortcutItems[i] = {path: node.evaluate("string(@Path)"), title: node.evaluate("string(@Title)"), category: node.evaluate("string(@Category)")};
    }
    
    // Next we query the xml to get the category items
    results = xml.evaluate("Shortcuts/Categories/Category");
  
    if (results.length == 0)
    {
      return;
    }
    
    for (i = 0; i < results.length; i++)
    {
      node = results.item(i);
      categories[i] = node.evaluate("string(@name)");
    }
  } else{
    // Xml file does not exist so we return having done nothing
    return;
  }
}

function addNewShortcut(type)
{
  switch(type)
  {
    case 'File':
      var file = chooseFile();
      if (file == null)
      {
        return;
      } else{
        createNewShortcutItem(file, filesystem.getDisplayName(file));
      }
      break;
    case 'Web':
      // Display an empty form
      createNewShortcutItem('','');
      break;
  }
}
  
function removeShortcut(shortcutID)
{
  var shortcutItems = getShortcuts(preferences.currentCategory.value);
  var strPath = shortcutItems[shortcutID].path;
  
  var results = xmlData.evaluate("Shortcuts/Shortcut[@Path = '" + strPath + "']");
  
  if (results.length != 0)
  {
    var scutNode = results.item(0);
    var rootNodeResults = xmlData.evaluate("Shortcuts");
    var rootNode = rootNodeResults.item(0);
    rootNode.removeChild(scutNode);
  }
  updatePageDisplay();
}

function newTitleFormField(strTitle)
{
  var formField = new FormField();
  formField.name = "shortcutTitle";
  formField.type = "text";
  formField.title = "Title:";
  formField.defaultValue = strTitle;
  formField.description = "The title of the shortcut is the text that appears in the icon tooltip";
  return formField;
}

function newPathFormField(strPath)
{
  var formField = new FormField();
  formField.name = "shortcutPath";
  // if this is a url, display a plain text field
  if (strPath.substr(0, 4) == "http")
    formField.type = "text";
  else
    formField.type = "selector";
  formField.title = "Path:";
  formField.defaultValue = strPath;
  formField.description = "The path which this shortcut will launch, for web sites please include the http:// etc.";
  return formField;
}

function newCategoryFormField(strCategory)
{
  var formField = new FormField();
  formField.name = "shortcutCategory";
  formField.type = "popup";
  formField.option = getCategories();
  formField.title = "Category:";
  formField.defaultValue = strCategory;
  formField.description = "The category that this shortcut belongs to";
  return formField;
} 

function editShortcutProperties(shortcutID)
{
  // Get a array of shortcuts using the current category
  var shortcutItems = getShortcuts(preferences.currentCategory.value);
  
  // Get the properties of the shortcut requested
  var strPath = shortcutItems[shortcutID].path;
  var strTitle = shortcutItems[shortcutID].title;
  var strCategory = shortcutItems[shortcutID].category;
  
  // Present a form to the user where values can be edited
  // Array of form field items to be added to the form
  var formFields = Array();
    
  formFields[0] = newTitleFormField(strTitle);
  
  formFields[1] = newPathFormField(strPath);
  
  formFields[2] = newCategoryFormField(strCategory);
    
  formResults = form(formFields, "Edit Shortcut Properties", "Save", "Cancel");
  
  // If the formResults is not null then we do something
  if (formResults != null)
  {
    // To make changes to the shortcut we need to manipulate the xml
    // First we need to get the element that we want to work with
    try
    {
      print("Shortcuts/Shortcut[@Path = '" + strPath + "']")
      var results = xmlData.evaluate("Shortcuts/Shortcut[@Path = '" + strPath + "']");
    } catch (ex){
      print(ex);
    }
    if (results.length != 0)
    {
      try
      {
      var node = results.item(0);
      node.setAttribute("Title", formResults[0]);
      node.setAttribute("Path", formResults[1]);
      node.setAttribute("Category", formResults[2]);
      } catch(ex)
      {
        print(ex);
      }
      // Invoke a redisplay in case the shortcut category changed
      updatePageDisplay();
    } else{
      print("Could not retrieve the XML element!");
      return;
    }
  } else{
    return;
  }
}

function createNewShortcutItem(strPath, strTitle)
{ 
  // Get the categories list
  var categories = getCategories();
  
  // Array of form field items to be added to the form
  var formFields = Array();
  formFields[0] = newTitleFormField(strTitle);
  
  formFields[1] = newPathFormField(strPath);
  
  var newCatInd = 2;
  // If categories is not null, show existing categories
  if (categories != null)
  {
    formFields[2] = newCategoryFormField(preferences.currentCategory.value);
    newCatInd=3;
  }
  // We will include a text box to allow people to create new categories at the same time
  // as creating the new shortcut, particularly useful if the widget is run for the first
  // time and the categories don't yet exist!
  formFields[newCatInd] = new FormField();
  formFields[newCatInd].name = "shortcutCategory";
  formFields[newCatInd].type = "text";
  formFields[newCatInd].title = "Create Category:";
  formFields[newCatInd].defaultValue = "";
  formFields[newCatInd].description = "If you don't want to add to an existing category then you can create a new one.\n\nWARNING: If a value is added here then any selection made in the category drop-down will be ignored!";
  
  formResults = form(formFields, "Add Shortcut", "Add", "Cancel");
    
  if (formResults != null)
  {
    // Add a new item to the shortcuts xml data
    // The method of adding depends on whether we need to create a new category or not
    if (categories == null || formResults[newCatInd] != "")
    {
      // Create a new category node
      var categoryNode = xmlData.createElement("Category");
      categoryNode.setAttribute("Name", formResults[newCatInd]);
      // Get the Categories node and add the new node to it
      var results = xmlData.evaluate("Shortcuts/Categories");
      if (results.length == 0)
      {
        // Create a new categories element
        var rootResults = xmlData.evaluate("Shortcuts");
        var rootNode = rootResults.item(0);
        var catsNode = xmlData.createElement("Categories");
        rootNode.appendChild(catsNode);
        catsNode.appendChild(categoryNode);
      } else{
        results.item(0).appendChild(categoryNode);
      } 
      var strCategory = formResults[newCatInd];
      preferences.currentCategory.value = strCategory;
    }  else{
      //existing category selected
      var strCategory = formResults[2];
    }
        
    var shortcutNode = xmlData.createElement("Shortcut");
    shortcutNode.setAttribute("Path", formResults[1]);
    shortcutNode.setAttribute("Title", formResults[0]);
    shortcutNode.setAttribute("Category", strCategory);
    var results = xmlData.evaluate("Shortcuts");
    results.item(0).appendChild(shortcutNode);
    
    updatePageDisplay();
  }
  else {
    // Do nothing and return to caller
    return;
  }
}

function saveShortcutData()
{
  filesystem.writeFile(xmlPath, xmlData.toXML());
}

function manageCategories(action)
{
  // Get an array of existing categories
  var categories = getCategories();
  
  switch(action)
  {
    case "Add":
      var formFields = Array();
      
      formFields[0] = new FormField();
      formFields[0].name = "newCategory";
      formFields[0].type = "text";
      formFields[0].title = "Name:";
      formFields[0].defaultValue = "";
      formFields[0].description = "The name to give your new category";
      
      formResults = form(formFields, "Add Category", "Add", "Cancel");
      
      if (formResults != null)
      {
        // Create a new category node
        var categoryNode = xmlData.createElement("Category");
        categoryNode.setAttribute("Name", formResults[0]);
        // Get the Categories node and add the new node to it
        var results = xmlData.evaluate("Shortcuts/Categories");
        if (results.length == 0)
        {
          // The shortcut categories list does not exist so we need to create it
          var rootResults = xmlData.evaluate("Shortcuts");
          var rootNode = rootResults.item(0);
          var catsNode = xmlData.createElement("Categories");
          // Add the new category element to the categories element
          catsNode.appendChild(categoryNode);
          // Add the categories node to the root element
          rootNode.appendChild(catsNode);
        } else{
          results.item(0).appendChild(categoryNode);
        }
      }
      preferences.currentCategory.value = formResults[0];
      updatePageDisplay();
      break;
    case "Remove":
      var formFields = Array();
      
      formFields[0] = new FormField();
      formFields[0].name = "newCategory";
      formFields[0].type = "popup";
      formFields[0].option = categories;
      formFields[0].title = "Name:";
      formFields[0].defaultValue = "";
      formFields[0].description = "Select which category you would like to remove";
      
      formResults = form(formFields, "Remove Category", "Remove", "Cancel");
      
      if (formResults != null)
      {
        // Remove the selected category node from the xml
        try
        {
          var results = xmlData.evaluate("Shortcuts/Categories/Category[@Name = '" + formResults[0] + "']");
          var catNode = results.item(0);
          results = xmlData.evaluate("Shortcuts/Categories");
          var catsNode = results.item(0);
          catsNode.removeChild(catNode);
        } catch(ex){
          print(ex);
        }
        
        var shortcutItems = getShortcuts(formResults[0]);
        
        if (shortcutItems != null)
        {
          var formFields2 = Array();
      
          formFields2[0] = new FormField();
          formFields2[0].name = "removeShortcuts";
          formFields2[0].type = "checkbox";
          formFields2[0].title = "Delete All Shortcuts";
          formFields2[0].defaultValue = 1;
          formFields2[0].description = "If the checkbox is ticked then all shortcuts related to the category will be removed and any option selected from the drop list below will be ignored";
      
          var items = Array();
          // Refresh the categories array to take into account the one we removed
          categories = getCategories();
          if (categories != null)
          {
            formFields2[1] = new FormField();
            formFields2[1].name = "moveCategory";
            formFields2[1].type = "popup";
            formFields2[1].option = categories;
            formFields2[1].title = "Name:";
            formFields2[1].defaultValue = "";
            formFields2[1].description = "Select which category you would like to move the shortcuts to";
          } else{
            formFields2[1] = new FormField();
            formFields2[1].name = "createCategory";
            formFields2[1].type = "text";
            formFields2[1].title = "Name:";
            formFields2[1].defaultValue = "";
            formFields2[1].description = "No categories exist so create a new one here.";
          }
      
          formResults2 = form(formFields2, "Move/Delete Shortcuts", "Move/Delete", "Cancel");
      
          if (formResults2 != null)
          {
            switch(formResults2[0])
            {
              case "0":
                if (formFields2[1].name == "createCategory")
                {
                  var bNew = true;
                } else{
                  var bNew = false;
                }
                if(formResults2[1] == "")
                {
                  alert("No new cateogry created, so shortcuts will be deleted!");
                  deleteShortcuts(formResults[0]);
                } else{
                  moveShortcuts(formResults[0], formResults2[1], bNew);
                }
                break;
              case "1":
                deleteShortcuts(formResults[0]);
                break;
            }
          }       
        } else{
          // We need to do an update so that the category information changes
          var categories = getCategories();
          if (categories != null)
          {
            preferences.currentCategory.value = categories[0];
          } else{
            preferences.currentCategory.value = "empty";
          }
          // Update the display       
          updatePageDisplay();
      }
      break;
    }
  }
}

function moveShortcuts(oldCategory, newCategory, createNew)
{
  shortcutItems = getShortcuts(oldCategory);
  if (shortcutItems != null)
  {
    if (createNew == true)
    {
      if (newCategory == "")
      {
        var result = prompt("A name must be provided for the new category!", "", "Provide Name For Category", "Continue", "Cancel");
        if (result != null)
        {
          newCategory = result
        } else{
          alert("No new category can be created, no name specified!");
          return;
        }
      }
      // Create a new category node
      var categoryNode = xmlData.createElement("Category");
      categoryNode.setAttribute("Name", newCategory);
      // Get the Categories node and add the new node to it
      var results = xmlData.evaluate("Shortcuts/Categories");
      if (results.length == 0)
      {
        // The shortcut categories list does not exist so we need to create it
        var rootResults = xmlData.evaluate("Shortcuts");
        var rootNode = rootResults.item(0);
        var catsNode = xmlData.createElement("Categories");
        // Add the new category element to the categories element
        catsNode.appendChild(categoryNode);
        // Add the categories node to the root element
        rootNode.appendChild(catsNode);
      } else{
        results.item(0).appendChild(categoryNode);
      }
    }
    
    for (i = 0; i < shortcutItems.length; i++)
    {
      try{
        var results = xmlData.evaluate("Shortcuts/Shortcut[@Path = '" + shortcutItems[i].path + "']");
        var node = results.item(0);
        node.setAttribute("Category", newCategory);
      } catch(ex){
        print(ex);
      }
    }
    preferences.currentCategory.value = newCategory;
    updatePageDisplay();
  }
}

function deleteShortcuts(category)
{
  var shortcutItems = getShortcuts(category);
  
  if (shortcutItems != null)
  {
    for (i = 0; i < shortcutItems.length; i++)
    {
      // First we need to get the shortcuts node
      var results = xmlData.evaluate("Shortcuts");
      var scutsNode = results.item(0);
      
      // Next we get the node for the shortcut to be removed
      results = xmlData.evaluate("Shortcuts/Shortcut[@Path = '" + shortcutItems[i].path + "']");
      var scutNode = results.item(0);
      
      // Remove the shortcut node from the xml
      scutsNode.removeChild(scutNode);
    }
    // We need to do an update so that the category information changes
    var categories = getCategories();
    if (categories != null)
    {
      print("Setting categories to first");
      preferences.currentCategory.value = categories[0];
    } else{
      print("Setting categories to nothing");
      preferences.currentCategory.value = "empty";
    }
    // Update the display       
    updatePageDisplay();
  }
}

function getShortcuts(category)
{
  // Create an array to hold the shortcuts
  var items = Array();
  
  // Get all the shortcuts that match the current category
  // A special check added here at v1.1 to allow the selection of all shortcuts
  if (category == "All Shortcuts")
  {
    // Select all shortcut items
    var results = xmlData.evaluate("Shortcuts/Shortcut");
  } else{
    // Query the xml data for the shortcuts that match the category specified
    var results = xmlData.evaluate("Shortcuts/Shortcut[@Category = '" + category + "']");
  }
  
  // Check if the query returns any results
  if (results.length == 0)
  {
    // No items found so return a null value for items
    return null;
  } else{
    // There are items so we need to populate the array with shortcut objects
    for (i = 0; i < results.length; i++)
    {
      var node = results.item(i);
      items[i] = {path: node.evaluate("string(@Path)"), title: node.evaluate("string(@Title)"), category: node.evaluate("string(@Category)")};
    }
    
    // Return the populated array
    return items;
  }
}

function getCategories()
{
  // Create an array to hold the category items
  var items = Array();
  
  // Query the xml for all the categories
  var results = xmlData.evaluate("Shortcuts/Categories/Category");
  
  // Check if the query returns any results
  if (results.length == 0)
  {
    // No results so return null
    return null;
  } else{
    // There are some results so populate the array and send it back
    for (i = 0; i < results.length; i++)
    {
      var node = results.item(i);
      items[i] = node.evaluate("string(@Name)");
    }
    return items;
  }
}

function changeCategory(category)
{
  // Set the preference to the new category
  preferences.currentCategory.value = category
  
  // Reset the current page to 0
  curPage = 0;
  
  // Call the display process
  updatePageDisplay();
}
